/*
 * Developed by Hillstone Software. http://www.hillstone-software.com
 * Socket interface layer
 */
#ifndef _HS_SOCK_IF_H
#define _HS_SOCK_IF_H

#include "..\HsIcmp\hsicmp_if.h"

/* return codes to user */
typedef enum
{
	HS_SOCK_RC_FAIL = 0,				// general failure, no additional information
	HS_SOCK_RC_OK,						// success
	HS_SOCK_RC_INITIALIZED,				// already initialised
	HS_SOCK_RC_PARAM,					// invalid parameters
	HS_SOCK_RC_NOT_INIT,				// socket layer not initialised
	HS_SOCK_RC_SOCKINIT_FAILED,	 		// OS socket module initialisation failed
	HS_SOCK_RC_CREATE_FAILED,			// socket call to OS socket module failed
	HS_SOCK_RC_BIND_FAILED,				// bind call to OS socket module failed
	HS_SOCK_RC_NO_MEM,					// no free memory
	HS_SOCK_RC_CONN_FAILED,				// connect failed
	HS_SOCK_PENDING,					// operation in progress (non blocking mode)
	HS_SOCK_RC_ALR_LISTENING,			// already listening for incoming connections
	HS_SOCK_RC_SEND_PENDING,            // Buffer could not be sent, try again later	

	HS_SOCK_NUM_RC,
} hssock_rc_t;

/* indications to user */
typedef enum
{
	HS_SOCK_EV_DATA,					// data received on a socket into user buffer
	HS_SOCK_EV_CLOSED,					// socket sesison closed by network
	HS_SOCK_EV_CONNECTED,				// TCP outgoing socket connected
	HS_SOCK_EV_ACCEPTED,				// TCP incoming connection accepted
	HS_SOCK_EV_CONN_FAILED,				// TCP connect failed
	HS_SOCK_EV_DHCP_UPDATE,				// IP parameters acquired / changed by DHCP module
} hssock_ev_t;

// timer callback
typedef void hs_sock_timer_cb_t(void *arg);
typedef long hs_sock_timer_start_t(unsigned long timeout_ms, void *arg, hs_sock_timer_cb_t *cb);
typedef void hs_sock_timer_stop_t(long timer_han);
typedef void hs_sock_drv_tx_fn_t(unsigned char *pkt, int len);
typedef void hs_sock_log_fn_t(char *str);

// event callback
typedef long hs_sock_event_fn_t(long user_handle, int ev, long arg1, long arg2, long arg3, long arg4);

// Get random 32 bit number
typedef unsigned long hs_sock_get_rand32_t(void);

// callback used by HSTCP to get number of millisecond ticks since bootup
// Used for generating pseudo random initial sequence number
// if callback pointer not initialized, HSTCP will use default 
// pseudo random number generator to produce initial sequence numbers
typedef unsigned long hs_sock_get_ticks_fn_t(void);

typedef struct
{
	unsigned char 			*local_eth; 		// local MAC address

	int						dhcp_client;		// 1=Run DHCP client protocol to get IP address; 0=IP address is static

	unsigned char 			*source_ip_str;		// local interface IP address 
	unsigned char 			*gateway_ip_str;	// gateway router IP address
	unsigned char 			*network_mask;		// local network mask

	unsigned char			*dns_server_ip_str;	// DNS server ip address

	hs_sock_event_fn_t		*global_ev_fn;		// event callback for reporting events to application not specific to a socket context

	hs_sock_timer_start_t 	*timer_start_fn;	// start timer callback
	hs_sock_timer_stop_t 	*timer_stop_fn;		// stop timer callback
	hs_sock_drv_tx_fn_t 	*drv_tx_fn;			// ethernet transmit callback
	hs_sock_get_ticks_fn_t 	*get_ticks_fn;		// get number of milliseconds since bootup
	hs_sock_get_rand32_t	*get_rand32;		// get random 32 bit number
	hs_sock_log_fn_t 		*log_fn;			// logging function for debugging

	unsigned long			seed;				// initial seed used by HSTCP to initialize PRNG for producing initial sequece numbers

	hsicmp_event_fn_t		*icmp_event_fn;		// icmp event callback
} hssock_init_t;

/*
 * Initialise socket interface layer
 */
extern int HsSockInit(hssock_init_t *init);

/*
 * CleanUp socket interface layer
 */
extern void HsSockCleanUp(void);

/*
 * Create UDP session and associate local IP port with it
 */
extern int HsSockUdpOpen(long			usr_handle, 	// user supplied handle
					hs_sock_event_fn_t  *event_fn,		// event callback function
					unsigned short 		source_port,	// source port
					long 				*sock_handle);	// returned socket layer handle

/*
 * Create outgoing TCP session
 */
int HsSockTcpConnect(long 				usr_handle, 		// user handle
					hs_sock_event_fn_t	*event_fn,			// socket event callback
					unsigned char		*dest_ip,  			// remote ip address to connect to
					unsigned short		dest_port,			// remote port
					long 				*sock_handle);		// socket handle returned

/*
 * Start listening for incoming TCP connections on specified port
 */
int HsSockTcpListen(unsigned short 	port, 				// port to listen on
					long 			user_handle, 		// user handle
					hs_sock_event_fn_t	*event_fn,			// socket event callback
					long 			*listen_sock_han);	// listen socket handle returned

/*
 * Close a session
 */
extern int HsSockClose(long  sock_handle);

/*
 * Send packet over UDP session
 */
int HsSockUdpSendto(long				hssk_handle,	// socket layer handle
					unsigned long 		dest_ip, 		// remote end IP address
					unsigned short int 	dest_port,		// remote UDP port number
					unsigned char 		*buffer, 		// payload data
					unsigned short int	length);			// payload data length

/*
 * Send packet over TCP session
 */
int HsSockTcpSend(long				hssk_handle,	// socket layer handle
					unsigned char 	*packet_buf, 	// packet data
					int 			length,         // packet data length
					int             *txed_len);		// bytes actually transmitted

/*
 * Check whether "cp" is a valid ascii representation
 * of an Internet address and convert to a binary address.
 * Returns 1 if the address is valid, 0 if not.
 * This replaces inet_addr, the return value from which
 * cannot distinguish between failure and a local broadcast address.
 */
extern int HsSockInetAddr(unsigned  char *cp, unsigned long *addr);

/* convert unsigned long ip address to IP address string */
extern void HsSockInetNtoa(unsigned long ipl, unsigned char *ip_addr);

/* Get descriptive error string corresponding to HsSock error code */
extern unsigned char *HsSockGetRcString(int rc);

#endif // _HS_SOCK_IF_H

